#!/usr/bin/perl
 
use Getopt::Std;               # Kommandozeilen-Parameter-Erfasser
use LWP::Simple;               # WWW-Utility
use HTML::TreeBuilder;         # HTML-Parser
use Archive::Tar;              # Tar-Archivierer
use URI::URL;

sub info { print STDERR @_ if $opt_v; }   # Ausgabe Verbose-Modus
sub err  { print STDERR @_; }             # Fehler-Ausgabe

getopts("ef:ght:v");           # Kommandozeilen-Parameter erfassen

usage() if(defined $opt_h);    # Help-Option gesetzt
usage() unless grep {defined} ($opt_e, $opt_g);  # Extract oder Get

my $tar = Archive::Tar->new() if $opt_t;         # Tar-Objekt erzeugen

if(defined $opt_f) {              # URLs aus einer Datei ...
  push(@ARGV, $opt_f);            # Datei einfach an die Kommandozeile
                                  # anha^ngen.
  while(<>) { chop; push(@urls, $_); }
} else {
  push(@urls, @ARGV) || usage();  # oder URLs von der Kommandozeile
}

foreach $url (@urls) {
  info "# GET URL $url ... ";
  ($doc = LWP::Simple::get $url) || 
     do { err "FAILED on ($url)\n"; next };
  info "OK\n";

  if($opt_t) {                         # Nicht ausgeben, => tarfile
    my $path = URI::URL->new($url)->path;
    $path =~ s,/$,/index.html,g; $path =~ s,^/,,g;
    $tar->add_data($path, $doc);       # Daten ins Archiv
    next;
  }

  if($opt_g) { print "$doc"; next; }   # Dokument ausgeben

                                       # Links extrahieren
  my $tree = HTML::TreeBuilder->new->parse($doc);

                                       # <A>, <AREA> und <IMG>
  for (@{$tree->extract_links(qw/a area img/)}) {
    my $l = URI::URL->new($_->[0]);    # href-Attribut
    ($s = $l->abs($url)) =~ s/#.*//g;  # URL absolut, #.. raus
     print "$s\n" unless $links{$s}++; # Ausgeben falls neu
  }

    $tree->delete();                   # Parse-Baum lo^schen
}

if($opt_t) {
  $tar->write($opt_t);                 # Tarfile erzeugen
  info "$opt_t ready.\n";              # Meldung im verbose-Mode
}

sub usage {
###############################################################
    $0 =~ s,.*/,,g;           # Pfad entfernen

    print <<EOT;
usage: $0 -g [-f URLfile] [-t tarfile] URL ...  # get URLs
       $0 -e [-f URLfile] URL ...               # extract links
options:
       -h: help
       -v: verbose
EOT
    exit 1;
}
